function [muMV, deltaMV, gammaMV, alphaMV, betaMV, difference] = MNIG_EMestimation(x)
%MNIG_EMestimation estimates the multivariate normal-inverse Gaussian
%distribution parameters for a set of 'd' observation series, sorted in 
%colums in 'x'. This particular code estimates the STANDARDIZED version of 
%the distribution, with expectation zero and identity covariance

%Written by Vegard Medb and ystein Arvesen
%June 5th 2012
%Part of Master Thesis at NTNU, Norway



T = size(x, 1);
K = size(x, 2);
tol = 1e-6;

%Transpose data to be (T,K) instead of (K,T)
x = x';

%Initializing starting values
gammaMV = eye(K);
betaMV = zeros(K,1);
alphaHolder = zeros(K,1);
muMV = zeros(K,1);
deltaHolder = zeros(K,1);

for k = 1:K
    m = mean(x(k,:));
    v = var(x(k,:));
    s = skewness(x(k,:));
    ku = kurtosis(x(k,:));
        
    [alpha_uni, beta_uni, mu_uni, delta_uni] = nigpar(m,v,s,ku);
    alphaHolder(k,1) = alpha_uni;
    betaMV(k,1) = beta_uni;
    muMV(k,1) = mu_uni;
    deltaHolder(k,1) = delta_uni;
end

deltaMV = sum(deltaHolder)/K;

alphaMV = 0;
for k = 1:K
    alphaMV = alphaMV + sqrt(alphaHolder(k)^2 - betaMV(k)^2 + betaMV'*gammaMV*betaMV);
end
alphaMV = alphaMV/K;


%Start the EM algorithm untill difference < tol
difference = 10;
counter = 1;

while difference > tol
    
    %Save old parameters to compute the difference
    delta1 = deltaMV;
    mu1 = muMV;
    gamma1 = gammaMV;
    alpha1 = alphaMV;
    beta1 = betaMV;
    
    disp('Iteration number: ');
    disp(counter);
    disp('Difference is now: ');
    disp(difference);
    counter = counter + 1;
    

   
    zeta = zeros(T,1);
    phi = zeros(T,1);
    
    for t = 1:T
        q_temp = q_func(deltaMV, x(:,t), gammaMV, muMV);
        zeta(t,1) = q_temp*besselk((K-1)/2, alphaMV*q_temp)/(alphaMV*besselk((K+1)/2, alphaMV*q_temp));
        phi(t,1) = alphaMV*besselk((K+3)/2, alphaMV*q_temp)/(q_temp*besselk((K+1)/2, alphaMV*q_temp));    
    
    end
    
    
    lambda = sum(zeta)/T;
    ksi = sum(phi)/T;
    nu = 0;
    for t = 1:T
        nu = nu + (phi(t,1)-lambda^-1);
    end
    nu = T/nu;
    
    deltaMV = sqrt(nu);
    littleGamma = deltaMV/lambda;
    %Options for fmincon in beta-update
    options  =  optimset('fmincon');
    options  =  optimset(options , 'TolFun'      , 1e-006);
    options  =  optimset(options , 'MaxFunEvals' , 2000);
    options  =  optimset(options , 'Algorithm' , 'interior-point');

    gammaTemp = (littleGamma/deltaMV)*eye(K) + (littleGamma^2/(2*(betaMV'*betaMV)^2)*(-1+sqrt(1+4*(betaMV'*betaMV)/(deltaMV*littleGamma)))-littleGamma/(deltaMV*(betaMV'*betaMV)))*(betaMV*betaMV');
    gammaTemp = gammaTemp/(det(gammaTemp)^(1/K));
    betaStart = betaMV;
    betaMV = fmincon(@(beta_in) betaEstimator(gammaTemp, phi, x, lambda, beta_in, T, ksi), betaStart, [], [], [], [], [], [], @(beta_in) betacon(beta_in, alphaMV, gammaMV), options);

    gammaMV = (littleGamma/deltaMV)*eye(K) + (littleGamma^2/(2*(betaMV'*betaMV)^2)*(-1+sqrt(1+4*(betaMV'*betaMV)/(deltaMV*littleGamma)))-littleGamma/(deltaMV*(betaMV'*betaMV)))*(betaMV*betaMV');
    gammaMV = gammaMV/(det(gammaMV)^(1/K));
    
    alphaMV = sqrt(littleGamma^2 + betaMV'*gammaMV*betaMV);
    muMV =  - lambda*gammaMV*betaMV;
    
    
    
    diff1 = abs(alphaMV - alpha1);
    diff2 = abs(deltaMV - delta1);
    diff3 = max(abs(muMV - mu1));
    diff4 = max(abs(betaMV - beta1));
    diff5 = max(max(abs(gammaMV-gamma1)));
    difference = max([diff1 diff2 diff3 diff4 diff5]);
    
    
end

disp('Expectation: ');
disp(muMV + lambda*gammaMV*betaMV);
disp('Covariance matrix is: ');
disp(deltaMV*(alphaMV^2-betaMV'*gammaMV*betaMV)^-0.5*(gammaMV+(alphaMV^2-betaMV'*gammaMV*betaMV)^-1*(gammaMV*(betaMV*betaMV')*gammaMV')));
    

end