function [sigma mu nu g] = skewT_EMestimation(x)
%SKEWT_EMESTIMATION estimates the multivariate skewT distribution
%parameters for a set of 'd' observation series, sorted in colums in 'x'.
%This particular code estimates the STANDARDIZED version of the
%distribution, with expectation zero and identity covariance

%Written by Vegard Medb and ystein Arvesen
%June 5th 2012
%Part of Master Thesis at NTNU, Norway


T = size(x, 1);
d = size(x, 2);
tol = 1e-3; %Small tolerance -> accurate parameter estimates
x = x';

%Options for minimization
options  =  optimset('fsolve');
options  =  optimset(options , 'Display'     , 'off');
options  =  optimset(options , 'MaxFunEvals' , 5000);
options  =  optimset(options , 'MaxIter'     , 5000);
options  =  optimset(options , 'TolFun'      , 1e-12);
options  =  optimset(options , 'TolX'        , 1e-12);

%Initializing parameter estimates (first guess)
mu = mean(x,2);
sigma = eye(d);
nu = 15;
g = skewness(x')';

%Declare auxilliary variable
rho = zeros(1,T);

difference = 10;
k = 1;



while(difference > tol)
   
    disp('Iteration: ');
    disp(k);
    
    for t = 1:T
        rho(1,t) = ((x(:,t) - mu)'/sigma)*(x(:,t) - mu);
    end
    
    %Define, for simplicity;
    gsg=(g'/sigma)*g;
    ind = (nu+d)/2;
    rhoNu = rho + nu;
    sqrt_term = sqrt( rhoNu .* gsg );
    
    %Compute auxilliary variables and their averages
    theta = (rhoNu./gsg).^(-0.5).*besselk(ind+1, sqrt_term)./besselk(ind, sqrt_term);
    eta = (rhoNu./gsg).^(0.5).*besselk(ind-1, sqrt_term)./besselk(ind, sqrt_term);
    
    %Simple numerical derivative of bessel function wrt. nu:
    h = 0.0001;
    xi = 0.5.*log(rhoNu./gsg)  + ((besselk(-ind + h, sqrt_term) - besselk(-ind-h, sqrt_term))/(2*h))./ besselk(ind, sqrt_term);
    
    thetaBar = mean(theta);
    etaBar = mean(eta);
    xiBar = mean(xi);
    xBar = mean(x,2);
    
    sigmaOld = sigma;
    muOld = mu;
    gOld = g;
    nuOld = nu;
    
    gTemp = zeros(d,1);
    for t = 1:T
        gTemp = gTemp + theta(1,t)*(xBar - x(:,t));
    end
    g = (1/T).*gTemp/(thetaBar*etaBar-1);
   
    
    
    nu = fzero( @(nuTemp) (-psi( nuTemp/2) + log (nuTemp/2) + 1 - xiBar - thetaBar ), 10, options);
    mu = -g*(nu/(nu-2));
    sigma = ((nu-2)/nu)*(eye(d) - (g*g')*(2*nu^2/((nu-2)^2*(nu-4))));
    
    
    diff1 = abs(nuOld-nu);
    diff2 = max(abs(gOld-g));
    diff3 = max(abs(muOld-mu));
    diff4 = max(max(abs(sigmaOld-sigma)));
    difference = max([diff1 diff2 diff3 diff4]);

    k = k+1;
    disp('Difference: ');
    disp(difference);
    
end



disp('Expectation:');
disp(mu + g*nu/(nu-2));
disp('Cov');
disp(nu/(nu-2)*sigma + g*g'*2*nu^2/((nu-2)^2*(nu-4)));



end

