function [realprices] = priceSimulation(T, numscenarios, var_p)
%priceSimulation simulates 'numscenarios' scenarios of T price observations
%of power, gas and carbon prices. 

%Written by Vegard Medb and ystein Arvesen
%June 5th 2012
%Part of Master Thesis at NTNU, Norway


global gammas;
global determ;
global alpha;
global beta;
global ECconst;
global R;
global garchparams;
global firstprices;
global muMV;
global deltaMV;
global gammaMV;
global alphaMV;
global betaMV;
global powerSeasonality;
global gasSeasonality;

%Initializing and formatting
realprices = zeros(T, numscenarios, 5); 
D = size(firstprices,1);
Tstart = size(firstprices, 2);
T = T+Tstart;
laglength = size(gammas,1)/D;  

IGvar = zeros(T,numscenarios);

elCos3 = powerSeasonality(1, var_p);
elSin3 = powerSeasonality(2, var_p);
elCos1 = powerSeasonality(3, var_p);
elSin1 = powerSeasonality(4, var_p);

gasCos3 = gasSeasonality(1, var_p);
gasSin3 = gasSeasonality(2, var_p);
gasCos1 = gasSeasonality(3, var_p);
gasSin1 = gasSeasonality(4, var_p);
gasCos9 = gasSeasonality(5, var_p);


 
for m = 1:numscenarios

    logprices = zeros(D, T);
    logprices(:, 1:Tstart) = firstprices(:,:, var_p);
    
    Ht = zeros(D,D,T); %Conditional covariance matrix
    Ht(:,:,Tstart) = 0.01*eye(D);
    Zt = zeros(D,T); %Simulated iid(0,I) errors
    Ut = zeros(D,T); %Simulated residuals    
    Ut(:,Tstart) = 0.05;
    ht = zeros(D,T);
    
    %Pre-define 'deltas' and for simplicity of notation, 'lambda_IG' and
    %'mu_IG'
      
    deltas = zeros(D,T);
    lambda_IG = deltaMV(:, var_p);
    mu_IG = sqrt(lambda_IG/sqrt(alphaMV(:, var_p)^2 - betaMV(:, var_p)'*gammaMV(:, :, var_p)*betaMV(:, var_p)));        
    
    for t = Tstart+1:T
        %Generate conditional volatility and residuals
        for d = 1:D
            ht(d,t) = garchparams(d,:, var_p)*[1; Ut(d, t-1)^2; ht(d, t-1)];
        end
        Ht(:,:,t) = diag(ht(:,t).^0.5)*R(:, :, var_p)*diag(ht(:,t).^0.5);
        IGvar(t,m) = winvgrnd(mu_IG, lambda_IG, [1,1]);
        Zt(:,t) = muMV(:, var_p) + IGvar(t,m)*gammaMV(:, :, var_p)*betaMV(:, var_p) + sqrt(IGvar(t,m))*chol(gammaMV(:, :, var_p))*mvnrnd([0 0 0], eye(D))'; 
        Ut(:,t) = chol(Ht(:,:,t))*Zt(:,t);
        
        for d = 1:D
            Ut(d,t) = max(min(1.5, Ut(d,t)), -1.5);
        end
               
        %Compute co-integrated mean equation
        for l = 1:laglength
            deltas(:,t) = deltas(:,t) + gammas(3*l-2:3*l,:, var_p)*(logprices(:,t-l)-logprices(:,t-l-1));
        end
        deltas(:,t) = deltas(:,t) + alpha(:, var_p)*(beta(:, var_p)'*logprices(:,t-1) + ECconst(:, var_p));

        %Compute day-of-week effects
        weekday = mod(t,6);
        if(weekday == 0)
            deltas(:,t) = deltas(:,t) + determ(:, 6, var_p);    
        else
            deltas(:,t) = deltas(:,t) + determ(:, weekday, var_p);    
        end
        
        %Add VECM, GARCH and deterministic effects into prices             
        logprices(:,t) = logprices(:,t-1) + deltas(:,t) + Ut(:,t);    


    end
    
    %Computing seasonal components
    times = 1:T;
    seasonalPower = elCos3*cos((2*pi/365).*times)+ elSin3*sin((2*pi/365).*times) + elCos1*sin((2*pi/183).*times) + elSin1*sin((2*pi/183).*times);
    seasonalGas = gasCos3*cos((2*pi/365).*times)+ gasSin3*sin((2*pi/365).*times) + gasCos1*sin((2*pi/183).*times) + gasSin1*sin((2*pi/183).*times) + gasCos9*cos((2*pi/91).*times);
    logprices(1,:) = logprices(1,:) + seasonalPower;
    logprices(2,:) = logprices(2,:) + seasonalGas;
    
    %Converting logprices back to prices
    for d = 1:D
        realprices(:,m,d) = exp(logprices(d, Tstart+1:T));
        
    end
    

    
end

end


