function R = wgamrnd(a,b,varargin)
% 002 %WGAMRND Random matrices from a Gamma distribution.
% 003 %
% 004 %  CALL:  R = wgamrnd(a,b,sz);
% 005 %
% 006 %       a,b = parameters, a,b>0
% 007 %        sz = size(R)    (Default common size of a and b)
% 008 %             sz can be a comma separated list or a vector 
% 009 %             giving the size of R (see zeros for options).
% 010 %
% 011 % The random numbers are generated by rejection sampling.
% 012 %
% 013 % Example:
% 014 %   R = wgamrnd(2,1,1,100);
% 015 %   plot(R,'.')
% 016 %
% 017 % See also  wgaminv
% 018 
% 019 
% 020 % Tested on: matlab 5.3
% 021 % History:
% 022 % based on rgamma.m (stixbox) by  (c) Anders Holtsberg 10-05-2000.
% 023 % revised pab 23.10.2000
% 024 %  - added default b
% 025 %  - added comnsize, nargchk
% 026 %  - added greater flexibility on the sizing of R
% 027 %  - rejection sampling method modified to give any size of a and b
% 028 %    the recursive call in rgamma is replaced with a while loop.
% 029 %  - replaced inversion method with a modified version of 
% 030 %    rgamma from stixbox (Anders Holtsberg)
% 031 % The algorithm is a rejection method. The logarithm of the gamma 
% 032 % variable is simulated by dominating it with a double exponential.
% 033 % The proof is easy since the log density is convex!
% 034 % 
% 035 % Reference: There is no reference! Send me (Anders Holtsberg) an email
%  % if you can't  figure it out.
 
 error(nargchk(1,inf,nargin))
 if nargin<2||isempty(b), b=1; end
 
 if nargin<3,
   [errorcode a b] = comnsize(a,b);
 else
   [errorcode a b] = comnsize(a,b,zeros(varargin{:}));
 end
 
 if errorcode > 0,
   error('a and b must be of common size or scalar.');
 end
 
 %R = wgaminv(rand(size(a)),a,b); % slower
 %return
 R = zeros(size(a));
 
 ok = ((a>0) & (b>0));
 k = find(ok);
 if any(k),
   ak=a(k);
   y0 = log(ak)-1./sqrt(ak);
   c  = ak - exp(y0);
   c1 =(ak.*y0 - exp(y0));
   c2 = abs((y0-log(ak)));
   
   accept=k;  omit=[];
   while ~isempty(accept)
     ak(omit)=[];  c(omit) =[];
     c1(omit)=[];  c2(omit)=[];
     sz = size(ak);
     la = log(ak);
     y  = log(rand(sz)).*sign(rand(sz)-0.5)./c + la;
 
     f = ak.*y-exp(y) - c1;
     g = c.*(c2 - abs(y-la));
   
     omit = find((log(rand(sz)) + g) <= f);
     if ~isempty(omit)
       R(accept(omit)) = exp(y(omit));
       accept(omit)=[];
     end
   end % while
   R(k) = R(k).*b(k);
 end
   
 k1=find(~ok);
 if any(k1);
   tmp=NaN;
   R(k1)=tmp(ones(size(k1)));
 end
 return
end
